@@ -3,7 +3,9 @@ import logging
 import os
 
 from .event_stream_base import EventStreamBase
+
 from kafka import KafkaConsumer, KafkaProducer
+from kafka.vendor import six
 
 from multiprocessing import Process, Queue, current_process, freeze_support, Pool
 
@@ -28,28 +30,52 @@ from multiprocessing import Process, Queue, current_process, freeze_support, Poo
 # eventstreap processor process producer1, consumer2,
 
 class EventStreamConsumer(EventStreamBase):
-
     relation_type = ''
-    state = "raw"
+    state = "unlinked"
+    topics = False
+    consumer = False
+
     task_queue = Queue()
     process_number = 4
-    log = "a EventStreamConsumer " + str(id) + " "
+    log = "EventStreamConsumer " + str(id) + " "
 
-    def get_consumer(self):
+    def create_consumer(self):
         logging.warning(self.log + "rt: %s" % self.relation_type)
-        if not self.topic_name:
-            self.topic_name = self.get_topic_name(state=self.state, relation_type=self.relation_type)
-        # self.topic_name = 'tweets'
-        logging.warning(self.log + "get consumer for topic: %s" % self.topic_name)
-        return KafkaConsumer(self.topic_name, group_id=self.group_id,
-                             bootstrap_servers=self.bootstrap_servers, api_version=self.api_version, consumer_timeout_ms=self.consumer_timeout_ms)
 
+        if self.state == 'all':
+            self.topics = self.build_topic_list()
+
+        if isinstance(self.state, six.string_types):
+            self.state = [self.state]
+
+        if isinstance(self.relation_type, six.string_types):
+            self.relation_type = [self.relation_type]
+
+        if not self.topics:
+            self.topics = list()
+            for state in self.state:
+                for relation_type in self.relation_type:
+                    self.topics.append(self.get_topic_name(state=state, relation_type=relation_type))
+
+        # self.topic_name = 'tweets'
+        logging.warning(self.log + "get consumer for topic: %s" % self.topics)
+        # consumer.topics()
+        self.consumer = KafkaConsumer(group_id=self.group_id,
+                             bootstrap_servers=self.bootstrap_servers, api_version=self.api_version,
+                             consumer_timeout_ms=self.consumer_timeout_ms)
+
+        for topic in self.topics:
+            logging.warning(self.log + "consumer subscribe: %s" % topic)
+            self.consumer.subscribe(topic)
+
+        logging.warning(self.log + "consumer subscribed to: %s" % self.consumer.topics())
 
     def consume(self):
-        logging.warning(self.log + "Consumer 1")
+        logging.warning(self.log + "start consume")
         self.running = True
 
-        consumer = self.get_consumer()
+        if not self.consumer:
+            self.create_consumer()
 
         # Start worker processes
         # for i in range(self.process_number):
@@ -58,19 +84,22 @@ class EventStreamConsumer(EventStreamBase):
 
         while self.running:
             try:
-                for msg in consumer:
-                    logging.warning(self.log + 'msg in consumer ')
+                for msg in self.consumer:
+                    logging.debug(self.log + 'msg in consumer ')
                     # logging.warning('msg in consumer %s' % msg.value)
                     self.task_queue.put(json.loads(msg.value.decode('utf-8')))
 
             except Exception as exc:
-                consumer.close()
+                self.consumer.close()
                 logging.error(self.log + 'stream Consumer generated an exception: %s' % exc)
                 logging.warning(self.log + "Consumer closed")
                 break
 
         if self.running:
-            self.consume()
+            return self.consume()
+
+        pool.close()
+        logging.warning(self.log + "Consumer shutdown")
 
     def worker(self, queue):
         logging.debug(self.log + "working %s" % os.getpid())
@@ -86,6 +115,7 @@ class EventStreamConsumer(EventStreamBase):
         self.running = False
         logging.warning(self.log + 'stop running consumer')
 
+
 if __name__ == '__main__':
-    e = EventStreamConsumer()
+    e = EventStreamConsumer(1)
     e.consume()
